/* 
 * Copyright (c) 2011, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * */

/*
 *  ======== TransportNyquistSim.c ======== 
 */

/* XDC include files */
#include <xdc/std.h>
#include <xdc/runtime/Assert.h>
#include <xdc/runtime/Memory.h>

/* UIA include files */
#include <ti/uia/runtime/UIAPacket.h>
#include <ti/uia/runtime/Transport.h>

#define UDPHDRSIZE      8
#define IPHDRSIZE      20
#define MACHDRSIZE     14
#define TOTALHDR       (MACHDRSIZE + IPHDRSIZE + UDPHDRSIZE)

/* IPv4 header */
typedef struct {
    UInt8    verLen;
    UInt8    tos;
    UInt16   totalLen;
    UInt16   id;
    UInt16   flagOff;
    UInt8    ttl;
    UInt8    protocol;
    UInt16   checksum;
    UInt32   IpSrc;
    UInt32   IpDst;
    UInt8    options[1];
} IpHdr;

/* Must be supplied by the application */
extern UInt8  hostMACAddress[6];  
extern UInt8  hostIPAddress[4];  
extern UInt8  myMACAddress[6];   
extern UInt8  myIPAddress[4];     

/* Supplied in the PDK */
extern Int32 SendPacket (UInt8* txData, UInt16 numBytes); 

/* Only one thread with call send, so a single buffer is fine. */
#pragma DATA_ALIGN(TransportNyquistSim_outPacket, 16)
Bits8 TransportNyquistSim_outPacket[1500] = {
    0x00, 0x1E, 0x4F, 0xBB, 0xA9, 0x66,   /* Host MAC address             */    
    0x00, 0x01, 0x02, 0x03, 0x04, 0x05,   /* my MAC address               */
    0x08, 0x0,                            /* IP Type                      */
    0x45,                                 /* IP Version & Header Len      */
    0x0,                                  /* TOS Byte                     */
    0x0, 0x0,                             /* size filled in the send      */
    0x0, 0x0,                             /* IPv4 Identifier              */
    0x0,                                  /* Flags                        */
    0x0,                                  /* Fragmentation Offset         */
    0x5,                                  /* TTL                          */
    0x11,                                 /* UDP                          */    
    0x0, 0x0,                             /* checksum filled in the send  */
    146, 252, 161, 150,                   /* my IP address                */
    146, 252, 161, 150,                   /* Host IP address              */
    0x04, 0xd3,                           /* Source UDP port              */
    0x04, 0xd3,                           /* Destination UDP port         */
    0x0, 0x0,                             /* size filled in the send      */
    0x0, 0x0,                             /* No checksum                  */
};

/* Useful in debugging */
Int TransportNyquist_failedUDPPacket; 

/*
 *  ======== TransportNyquistSim_IPChecksum ========
 */
Void TransportNyquistSim_IPChecksum(IpHdr *pIpHdr)
{
    Int     tmp1;
    UInt16  *pw;
    UInt32  tSum = 0;

    /* Get header size in 4 byte chunks */
    tmp1 = pIpHdr->verLen & 0xF;

    /* Checksum field is NULL in checksum calculations */
    pIpHdr->checksum = 0;

    /* Checksum the header */
    pw = (UInt16 *)pIpHdr;
    do {
        tSum += (UInt32)*pw++;
        tSum += (UInt32)*pw++;
    } while( --tmp1 );
    tSum = (tSum&0xFFFF) + (tSum>>16);
    tSum = (tSum&0xFFFF) + (tSum>>16);
    tSum = ~tSum;

    /* Note checksum is Net/Host byte order independent */
    pIpHdr->checksum = (UInt16)tSum;
}

/*
 *  ======== TransportNyquistSim_init ========
 */
Void TransportNyquistSim_init()
{   
    TransportNyquist_failedUDPPacket = 0;     
    
    /* Set my addrs */
    memcpy (&TransportNyquistSim_outPacket[6], &myMACAddress, 6);
    memcpy (&TransportNyquistSim_outPacket[26], &myIPAddress, 4);    

    /* Set Host addrs  */
    memcpy (&TransportNyquistSim_outPacket[0], &hostMACAddress, 6);
    memcpy (&TransportNyquistSim_outPacket[30], &hostIPAddress, 4);        
}

/*
 *  ======== TransportNyquistSim_start ========
 */
Ptr TransportNyquistSim_start(UIAPacket_HdrType hdrType)
{ 
    return (NULL);
}

/*
 *  ======== TransportNyquistSim_recv ========
 */
SizeT TransportNyquistSim_recv(Ptr handle, UIAPacket_Hdr **packet, SizeT size)
{
    /* Incoming messages currently not supported */
    return (0);
}

/*
 *  ======== TransportNyquistSim_send ========
 */
Bool TransportNyquistSim_send(Ptr handle, UIAPacket_Hdr **packet)
{
    Int32 status; 
    SizeT eventSize;    
    Bool successFlag = TRUE;
    extern cregister volatile UInt DNUM;
    
    /* Fill in the source address value */
    UIAPacket_setSenderAdrs(*packet, DNUM);
    
    if (UIAPacket_getHdrType(*packet) == UIAPacket_HdrType_Msg) {                
        successFlag = FALSE;
    }
    else { 
        eventSize = UIAPacket_getEventLength(*packet);        
        
        /* Fill in the IP length field */
        TransportNyquistSim_outPacket[16] = 
            (Bits8)((eventSize + UDPHDRSIZE + IPHDRSIZE) >> 8);
        TransportNyquistSim_outPacket[17] = 
            (Bits8)(eventSize + UDPHDRSIZE + IPHDRSIZE);
        
        /* Compute the checksum */
        TransportNyquistSim_IPChecksum((IpHdr *)&(TransportNyquistSim_outPacket[14]));
    
        /* Fill in the UDP length field */
        TransportNyquistSim_outPacket[38] = 
            (Bits8)((eventSize + UDPHDRSIZE) >> 8);
        TransportNyquistSim_outPacket[39] = (Bits8)(eventSize + UDPHDRSIZE);
        
        /* UIA data */
        memcpy (&(TransportNyquistSim_outPacket[42]), *packet, eventSize);    
            
        /* Send the packet */
        status = SendPacket(TransportNyquistSim_outPacket, (eventSize + TOTALHDR));
        if (status != 0)
        {
            TransportNyquist_failedUDPPacket++;
            successFlag = FALSE;
        }
    }    
    
    return (successFlag);
}

/*
 *  ======== TransportNyquistSim_stop ========
 */
Void TransportNyquistSim_stop(Ptr handle)
{   
}

/*
 *  ======== TransportNyquistSim_exit ========
 */
Void TransportNyquistSim_exit()
{    
}

/*
 */
/*
 *  @(#) ti.uia.sysbios; 1, 0, 0, 0,6; 6-15-2011 11:41:53; /db/vtree/library/trees/uia/uia.git/src/ null
 */

